/* 
 *
 * Created on 9. Dezember 2010, 12:50
 * Copyright (C) 2010 Klaus Ramstöck klra67@freenet.de
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef ALEXTYPES_H
#define	ALEXTYPES_H
#include <limits>

// Define some basic types
typedef float AlexReal;
typedef int AlexInt;
typedef long AlexId;
typedef unsigned int AlexIndex;

/*! \brief AlexTriple is the base class for Nodes, Vectors and the like.
 *
 * An AlexTriple provides the functionality common to objects like Nodes,
 * Points, Vectors and the like.
 */

class AlexTriple{
public:
    AlexReal x,y,z;
    AlexTriple(AlexReal myX, AlexReal myY, AlexReal myZ){x=myX;y=myY;z=myZ;}
    AlexTriple(){x=0.; y=0.; z=0.;}
};


#define ALEX_INVALID_TRIPLE AlexTriple(std::numeric_limits<AlexReal>::quiet_NaN(),std::numeric_limits<AlexReal>::quiet_NaN(),std::numeric_limits<AlexReal>::quiet_NaN() )
#define IS_VALID_TRIPLE(t) (t.x != std::numeric_limits<AlexReal>::quiet_NaN())
#define ALEX_INVALID_ID (-1)
#define ALEX_INVALID_INDEX (std::numeric_limits<int>::max())
#define IS_VALID_ID(id) (id>=0)
#define IS_VALID_INDEX(idx) (idx != ALEX_INVALID_INDEX)
#endif	/* ALEXTYPES_H */

