/* 
 *
 * Created on 9. Dezember 2010, 13:39
 * Copyright (C) 2010 Klaus Ramstöck klra67@freenet.de
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
*/

#include <cstdlib>
#include <iostream>
#include <ostream>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "AlexTypes.h"
#include "AlexElements.h"
#include "AlexTria.h"
#include "Basic.h"


AlexTrias trias;
AlexNodes nodes;
AlexIndex count;

static std::map<AlexIndex, bool> randomMap;


// This is the mersenne twister as taken from Wikipedia. We dont use rand()
// because we need to get the same random numbers on all plattforms all the time.
// Random number quality is not important.
#define MT_MAX 0xffffffff

unsigned mersenne_twister(void) {
  const int N = 624;
  const int M = 397;
  const unsigned A[2] = { 0, 0x9908b0df };
  const unsigned HI = 0x80000000;
  const unsigned LO = 0x7fffffff;

  static unsigned y[N];
  static int index = N+1;

  if (index >= N) {
    if (index > N) {
       // initialisiere y mit Pseudozufallszahlen:
       unsigned r = 9, s = 3402;
       for (int i=0 ; i<N ; ++i) {
          r = 509845221 * r + 3;
          s *= s + 1;
          y[i] = s + (r >> 10);
       }
    }
    unsigned h;
    for (int k=0 ; k<N-M ; ++k) {
       h = (y[k] & HI) | (y[k+1] & LO);
       y[k] = y[k+M] ^ (h >> 1) ^ A[h & 1];
    }
    for (int k=N-M ; k<N-1 ; ++k) {
       h = (y[k] & HI) | (y[k+1] & LO);
       y[k] = y[k+(M-N)] ^ (h >> 1) ^ A[h & 1];
    }
    h = (y[N-1] & HI) | (y[0] & LO);
    y[N-1] = y[M-1] ^ (h >> 1) ^ A[h & 1];
    index = 0;
  }

  unsigned e = y[index++];
  // tempering:
  e ^= (e >> 11);
  e ^= (e << 7) & 0x9d2c5680;
  e ^= (e << 15) & 0xefc60000;
  e ^= (e >> 18);
  return e;
}

AlexIndex randInt(AlexIndex max){
    unsigned int mt;
    mt = mersenne_twister() % max;
    return mt;
}

AlexReal randReal(){
    return ((AlexReal) mersenne_twister()) / MT_MAX;
}

void prepareRandomIds(AlexIndex max){
    if(randomMap.size())
        randomMap.erase(randomMap.begin(), randomMap.end());
    for(AlexIndex count=0;;){
        AlexIndex id = randInt(10*max);
        if(randomMap.find(id)==randomMap.end()) {
            randomMap[id]=true;
            if(++count == max) return;
        }
    }
}
    
AlexIndex randomId(){
    assert(randomMap.size()>0);
    AlexIndex id = randomMap.begin()->first;
    randomMap.erase(randomMap.begin());
    return id;
}

AlexNode randomNode(){
    AlexNode myNode;
    myNode.id = randomId();

    myNode.pos.x = randReal();
    myNode.pos.y = randReal();
    myNode.pos.z = randReal();

    return myNode;
}

void populateNodes(AlexNodes &nodes, AlexIndex max){

    nodes.Nodes.reserve(max);
    for(AlexIndex i=0; i<max; i++){
        nodes.Nodes.push_back(randomNode());
    }
    nodes.sortNodes();
}

void populateTrias(AlexTrias &trias, AlexNodes &nodes, AlexIndex max){

    for(AlexIndex i=0; i<max; i++){
        AlexId rndId = randomId();
        trias.map[rndId].n1 = nodes.Nodes[randInt(nodes.Nodes.size())].id;
        trias.map[rndId].n2 = nodes.Nodes[randInt(nodes.Nodes.size())].id;
        trias.map[rndId].n3 = nodes.Nodes[randInt(nodes.Nodes.size())].id;
        trias.map[rndId].setPartId(1);
    }
}

int init_nodes(void){
    prepareRandomIds(count+1);
    return 0;
}

int clean_nodes(void){
    // Keep nodes for further tests
    return 0;
}

void test_nodes_creation(void){
     populateNodes(nodes, count);
     CU_ASSERT(count = nodes.Nodes.size());
}

void test_nodes_access(void){
    std::vector<AlexId> NIds;
    NIds.reserve(count);
    for(NodeIterator i = nodes.begin(); i!=nodes.end(); i++){
        NIds.push_back(i->id);
    }
    try{
        for(AlexIndex i=0; i<count; i++){
            AlexNode tmp;
            tmp = nodes.getNodeById(NIds[randInt(count)]);
            CU_ASSERT(IS_VALID_TRIPLE(tmp.pos));
            tmp = nodes[NIds[randInt(count)]];
            CU_ASSERT(IS_VALID_TRIPLE(tmp.pos));
        }
    }catch (AlexException e) {
        CU_ASSERT(false);
    }
}

void test_nodes_access_nonexisting_id(void){
     AlexNode tmp;

     try{
       tmp = nodes.getNodeById(count*10+1);
       CU_ASSERT(false);
   } catch(AlexException e){
   }

   try{
       tmp = nodes.getNodeById(ALEX_INVALID_ID);
       CU_ASSERT(false);
   } catch(AlexException e){
   }
}

int init_trias(void){
    prepareRandomIds(count+1);
    return 0;
}

int clean_trias(void){
    trias.map.clear();
    return 0;
}

void test_trias_creation(void){
     populateTrias(trias, nodes, count);
     CU_ASSERT(count = trias.map.size());
}

void test_trias_access(void){
    std::vector<AlexId> Ids;
    Ids.reserve(count);
    for(TriasIterator i = trias.begin(); i!=trias.end(); i++){
        Ids.push_back(i->first);
    }
    AlexReal dmy=0;
    try{
        for(AlexIndex i=0; i<count; i++){
            AlexTria tmp;
            tmp = trias[Ids[randInt(count)]];
            dmy+=tmp.n1/count;
        }
    } catch(AlexException e){
        CU_ASSERT(false);
    }
    CU_ASSERT(dmy>=0);
}

void test_trias_access_nonexisting_id(void){
   AlexTria tmp;

   try{
       tmp = trias[count*10+1];
       CU_ASSERT(false);
   } catch(AlexException e){
   }

   try{
       tmp = trias[ALEX_INVALID_ID];
       CU_ASSERT(false);
   } catch(AlexException e){
   }

}

void test_trias_access_node_id(void){
   AlexTria tmp;

   std::vector<AlexId> MyElements = trias.getElementsWithNode(nodes.Nodes[0].id);
   unsigned int sz = MyElements.size();
   CU_ASSERT(sz==2 && count == 1000);
   MyElements = trias.getElementsWithNode(nodes.Nodes[1].id);
   sz = MyElements.size();
   CU_ASSERT(sz==5 && count == 1000);
   MyElements = trias.getElementsWithNode(nodes.Nodes[2].id);
   sz = MyElements.size();
   CU_ASSERT(sz==7 && count == 1000);
   MyElements = trias.getElementsWithNode(nodes.Nodes[3].id);
   sz = MyElements.size();
   CU_ASSERT(sz==2 && count == 1000);
}

void test_trias_access_part_id(void){
   AlexTria tmp;

   std::vector<AlexId> MyElements = trias.getElementsWithPart(1);
   unsigned int sz = MyElements.size();
   CU_ASSERT(sz==count);
}


int main(int argc, char** argv) {

    // Unbuffer stdout so we immediately see messages
    setvbuf(stdout, NULL, _IONBF, 0);
    count = 1000;

    // Setup CUnit
    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
   if (CUE_SUCCESS != CU_initialize_registry())
      return CU_get_error();

   /* add a suite to the registry */
   pSuite = CU_add_suite("Nodes", init_nodes, clean_nodes);
   if (NULL == pSuite) {
      CU_cleanup_registry();
      return CU_get_error();
   }
   // Add tests for nodes
    if ((NULL == CU_add_test(pSuite, "node creation", test_nodes_creation)) ||
        (NULL == CU_add_test(pSuite, "node access by id", test_nodes_access)) ||
        (NULL == CU_add_test(pSuite, "node access, non-existing id", test_nodes_access_nonexisting_id)))
   {
      CU_cleanup_registry();
      return CU_get_error();
   }

   /* add a suite to the registry */
   pSuite = CU_add_suite("Trias", init_trias, clean_trias);
   if (NULL == pSuite) {
      CU_cleanup_registry();
      return CU_get_error();
   }
   // Add tests for nodes
    if ((NULL == CU_add_test(pSuite, "tria creation", test_trias_creation)) ||
        (NULL == CU_add_test(pSuite, "tria access by id", test_trias_access)) ||
        (NULL == CU_add_test(pSuite, "tria access, nonexisting id", test_trias_access_nonexisting_id)) ||
        (NULL == CU_add_test(pSuite, "tria access by node id", test_trias_access_node_id)) ||
        (NULL == CU_add_test(pSuite, "tria access by part id", test_trias_access_part_id)))
   {
      CU_cleanup_registry();
      return CU_get_error();
   }

   CU_basic_set_mode(CU_BRM_VERBOSE);
   CU_basic_run_tests();
   CU_cleanup_registry();

}


