!
!> \file ListOps.f95
!! \brief Operations on Lists
!
!> Modul: Listops
!> \author Klaus Ramstöck klra67@freenet.de
!> \date 26.03.2011
! (C) 2011 Klaus Ramstöck klra67@freenet.de
!
! This program is free software; you can redistribute it and/or modify it under the terms
!  of the GNU General Public License as published by the Free Software Foundation; either
!  version 3 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
! without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
! See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this program;
!  if not, see <http://www.gnu.org/licenses/>.
!
MODULE ListOps
    USE AlexTypes
    IMPLICIT NONE
    PRIVATE
    PUBLIC ListFind, ListSort
contains
    !
    !> Find id in sorted list of length fill with access function getter
    INTEGER(AlexIdKind) FUNCTION ListFind(id, fill, getter)
        INTEGER(AlexIdKind), intent(in) :: id, fill
        INTERFACE
            INTEGER(AlexIdKind) FUNCTION getter(i)
                USE AlexTypes
                INTEGER(AlexIdKind), intent(in) :: i
            END FUNCTION getter
        END INTERFACE
        integer nfind, ntry
        common /listsort/ nfind, ntry
        INTEGER(AlexIdKind) :: lo, hi, i, gid, hiid, loid
        nfind = nfind + 1
        lo = 1
        hi = fill
        hiid = getter(hi)
        loid = getter(lo)
        DO
            ntry = ntry + 1
            if(hiid .eq.id) then
                listfind = hi
                return
            end if
            if(loid .eq.id) then
                listfind = lo
                return
            end if
            if(loid.eq.hiid) then
                listfind = invalidid
                return
            end if
            i = (hi*loid +(id*hiid)*lo+hi*id)/(loid-hiid)
            if(i.lt. lo .or. i.gt.hi) i=(hi+lo)/2
            gid = getter(i)
            IF (gid .EQ. id) THEN
                listfind = i
                RETURN
            END IF
            IF (gid .LT. id) THEN
                lo = i
                loid = gid
            ELSE
                hi = i
                hiid = gid
            END IF
            if(hi-lo<5)then
                ! linear search
                do i=lo, hi
                    if(getter(i) .eq. id) then
                        listfind = i
                        return
                    end if
                end do
                listfind = invalidId
                return
            END IF
        ENDDO

    END FUNCTION ListFind

    !> Sort list of length fill with access getter and swapper swap
    !> Heapsort, we expect sorted input
    SUBROUTINE ListSort(fill, getter, swap)
        INTEGER(AlexIdKind), intent(in) :: fill
        INTERFACE
            INTEGER(AlexIdKind) FUNCTION getter(i)
                USE AlexTypes
                INTEGER(AlexIdKind), intent(in) :: i
            END FUNCTION getter
            SUBROUTINE swap(i, j)
                USE AlexTypes
                INTEGER(AlexIdKind), intent(in) :: i, j
            END SUBROUTINE swap
        END INTERFACE
        INTEGER(AlexIdKind) :: n, v
        !
        ! build heap
        do v = fill/2, 1, -1
            call downheap(v, fill, getter, swap);
        end do
       !
        do n = fill, 2, -1
            call swap(1_AlexIdKind, n)
            call downheap(1_AlexIdKind, n-1, getter, swap)
        end do
    END SUBROUTINE ListSort

    SUBROUTINE downheap(vv,n, getter, swap)
        INTEGER(AlexIdKind), intent(in) :: vv,n
        INTERFACE
            INTEGER(AlexIdKind) FUNCTION getter(i)
                USE AlexTypes
                INTEGER(AlexIdKind), intent(in) :: i
            END FUNCTION getter
            SUBROUTINE swap(i, j)
                USE AlexTypes
                INTEGER(AlexIdKind), intent(in) :: i, j
            END SUBROUTINE swap
        END INTERFACE
        INTEGER(AlexIdKind) :: w,v
        v=vv
        w = 2 * (v-1) + 2
        do while (w .le. n)
            if (w + 1 .le. n) then
                if (getter(w + 1) .gt. getter(w)) then
                    w=w+1;
                end if
            end if

            if (getter(v) .ge. getter(w)) return

            call swap(v, w)
            v = w
            w = 2 * (v-1) + 2
        end do

    END SUBROUTINE downheap

    END MODULE ListOps
