!     
! File:   ModNodes.f95
! Author: klaus
!
! Created on 21. März 2011, 14:53
! (C) 2011 Klaus Ramstöck klra67@freenet.de
!
! This program is free software; you can redistribute it and/or modify it under the terms
!  of the GNU General Public License as published by the Free Software Foundation; either
!  version 3 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
! without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
! See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this program;
!  if not, see <http://www.gnu.org/licenses/>.

MODULE ModNodes
    USE AlexTypes
    USE ModVector
    USE ModElements
    USE ListOps
    IMPLICIT NONE
    PRIVATE

    PUBLIC :: Node, Nodes
    PUBLIC :: InitializeNodes, InsertNode, getNode, distV

    TYPE Node
        INTEGER(KIND = AlexIdKind) :: Id
        TYPE(Vector) :: p
        TYPE(ElementP), pointer :: referer(:)
    END TYPE Node

    TYPE Nodes
        INTEGER(KIND = AlexIdKind) :: Fill
        TYPE(Node), pointer :: nodes(:)
    END TYPE Nodes

    TYPE(Nodes), POINTER :: getterGlobal

CONTAINS

    SUBROUTINE InitializeNodes(this, osz, stat)
        TYPE(Nodes), intent(inout) :: this
        TYPE(Node), pointer :: dummy
        INTEGER(AlexIdKind), intent(in), Optional :: osz
        INTEGER, intent(out), Optional :: stat
        INTEGER(AlexIdKind) :: i, sz
        this % fill = 0
        sz = DefaultTableSize
        if (present(osz)) sz = osz
        nullify(this % nodes)
        if (present(stat)) then
            ALLOCATE(this % nodes(sz), stat = stat)
        ELSE
            ALLOCATE(this % nodes(sz))
        end if
    END SUBROUTINE InitializeNodes

    SUBROUTINE InsertNode(this, n, stat)
        TYPE(Nodes), intent(inout) :: this
        Type(Node), intent(in) :: n
        INTEGER, intent(out), Optional :: stat
        integer :: i
        if (this % fill .eq. size(this % nodes)) then
            call GrowNodes(this, stat)
        END IF
        DO i = 1, this % fill
            if (this % nodes(i) % Id > n % id) exit
        END DO
        if (i .le. this % fill) call shiftNodes(this, i)
        this % nodes(i) = n
    END SUBROUTINE InsertNode

    FUNCTION getNode(this, id, stat) result(n)
        TYPE(Nodes), intent(in), target :: this
        INTEGER(kind = AlexIdKind), intent(in) :: id
        INTEGER, intent(out), Optional :: stat
        TYPE(Node) :: n
        INTEGER(kind = AlexIdKind) :: i
        getterGlobal => this
        i = ListFind(id, this % fill, getter)
        if (i .eq. InvalidId) go to 8000
        n = this % nodes(i)
        goto 9000
        8000 continue
        if (present(stat)) then
            stat = ENotFound
        ELSE
            print *, "Node ", id, " not found in ", this % fill, " Nodes."
            stop
        end if
        9000 continue
    END FUNCTION getNode

    FUNCTION distV(n1, n2) result(v)
        type(Node), intent(in) :: n1, n2
        Type(Vector) :: v
        v = n2 % p;
        call sub(v, n1 % p)
    END FUNCTION distV

    SUBROUTINE GrowNodes(this, stat)
        TYPE(Nodes), intent(inout) :: this
        INTEGER, intent(out), Optional :: stat
        TYPE(Node), pointer :: newNodes(:)
        INTEGER :: newSize, i
        newSize = size(this % nodes) * 2
        if (present(stat)) then
            allocate(newNodes(newSize), stat = stat)
        else
            allocate(newNodes(newSize))
        endif
        DO i = 1, this % fill
            newNodes(i) = this % Nodes(i)
        END DO
        if (present(stat)) then
            deallocate(this % nodes, stat = stat)
        else
            deallocate(this % nodes)
        endif
        this % nodes => newNodes
    END SUBROUTINE GrowNodes

    SUBROUTINE ShiftNodes(this, p)
        TYPE(Nodes), intent(inout) :: this
        INTEGER, intent(in) :: p
    END SUBROUTINE ShiftNodes

    INTEGER(AlexIdKind) FUNCTION getter(i)
        INTEGER(AlexIdKind), intent(in) :: i
        getter = getterGlobal % nodes(i) % id
    END FUNCTION getter

END MODULE ModNodes