!
!> \file ModVector.f95
!! \brief 3D Vector modul
!
!> Modul: ModVector
!> \author Klaus Ramstöck klra67@freenet.de
!> \date 26.03.2011
! (C) 2011 Klaus Ramstöck klra67@freenet.de
!
! This program is free software; you can redistribute it and/or modify it under the terms
!  of the GNU General Public License as published by the Free Software Foundation; either
!  version 3 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
! without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
! See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this program;
!  if not, see <http://www.gnu.org/licenses/>.

MODULE ModVector
    USE AlexTypes
    IMPLICIT NONE
    PRIVATE

    PUBLIC:: Vector
    PUBLIC:: FNAdd, Add, FNSub, Sub, Dot, Dist, FNCross, Norm, FNNormalize, Normalize, Scale

    TYPE Vector
        REAL(AlexRealKind) :: x, y, z
    END TYPE Vector
CONTAINS
    FUNCTION FNAdd(this, b) result(V)
        TYPE(Vector), intent(inout) :: this
        TYPE(Vector), intent(in) :: b
        TYPE(Vector) :: V
        this % x = this % x + b % x
        this % y = this % y + b % y
        this % z = this % z + b % z
        v = this
    END FUNCTION FNAdd

    SUBROUTINE Add(this, b)
        TYPE(Vector), intent(inout) :: this
        TYPE(Vector), intent(in) :: b
        this % x = this % x + b % x
        this % y = this % y + b % y
        this % z = this % z + b % z
    END SUBROUTINE Add

    FUNCTION FNSub(this, b) result(V)
        TYPE(Vector), intent(inout) :: this
        TYPE(Vector), intent(in) :: b
        TYPE(Vector) :: V
        this % x = this % x - b % x
        this % y = this % y - b % y
        this % z = this % z - b % z
        v = this
    END FUNCTION FNSub

    SUBROUTINE Sub(this, b)
        TYPE(Vector), intent(inout) :: this
        TYPE(Vector), intent(in) :: b
        this % x = this % x - b % x
        this % y = this % y - b % y
        this % z = this % z - b % z
    END SUBROUTINE Sub

    FUNCTION Dot(this, b) result(d)
        TYPE(Vector), intent(in) :: this, b
        REAL(AlexRealKind) :: d
        d = this % x * b % x + this % y * b % y + this % z * b % z
    END FUNCTION Dot

    FUNCTION FNCross(this, b) result(c)
        TYPE(Vector), intent(in) :: this, b
        TYPE(Vector) :: c
        c % x = this % y * b % z - this % z * b % y
        c % y = -(this % x * b % z - this % z * b % x)
        c % z = this % x * b % y - this % y * b % x
    END FUNCTION FNCross

    FUNCTION Dist(this, b) result(d)
        TYPE(Vector), intent(in) :: this, b
        REAL(AlexRealKind) :: d
        d = sqrt((this % x - b % x)**2 + (this % y - b % y)**2 + (this % z - b % z)**2)
    END FUNCTION Dist

    FUNCTION Norm(this) result(len)
        TYPE(Vector), intent(in) :: this
        REAL(AlexRealKind) :: len
        len = sqrt(this % x **2 + this % y **2 + this % z **2)
    END FUNCTION Norm

    FUNCTION FNNormalize(this) result(n)
        TYPE(Vector), intent(inout) :: this
        TYPE(Vector) :: n
        REAL(AlexRealKind) :: len
        len = sqrt(this % x **2 + this % y **2 + this % z **2)
        IF (len .gt. 0.) THEN
            this % x = this % x / len
            this % y = this % y / len
            this % z = this % z / len
        END IF
        n = this
    END FUNCTION FNNormalize

    SUBROUTINE Normalize(this)
        TYPE(Vector), intent(inout) :: this
        REAL(AlexRealKind) :: len
        len = sqrt(this % x **2 + this % y **2 + this % z **2)
        IF (len .gt. 0.) THEN
            this % x = this % x / len
            this % y = this % y / len
            this % z = this % z / len
        END IF
    END SUBROUTINE Normalize

    FUNCTION Scale(this, s) result(n)
        TYPE(Vector), intent(inout) :: this
        REAL(AlexRealKind), intent(in) :: s
        TYPE(Vector) :: n
        this % x = this % x * s
        this % y = this % y * s
        this % z = this % z * s
        n = this
    END FUNCTION Scale

END MODULE ModVector