!     
! File:   FUnit.f95
! Author: klaus
!
! Created on 22. März 2011, 09:06
! (C) 2011 Klaus Ramstöck klra67@freenet.de
!
! This program is free software; you can redistribute it and/or modify it under the terms
!  of the GNU General Public License as published by the Free Software Foundation; either
!  version 3 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
! without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
! See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this program;
!  if not, see <http://www.gnu.org/licenses/>.

MODULE FUnit
    IMPLICIT NONE
    PRIVATE

    INTEGER, PARAMETER :: int4 = selected_int_kind(6)
    INTEGER, PARAMETER :: int8 = selected_int_kind(15)
    INTEGER, PARAMETER :: real4 = selected_real_kind(6)
    INTEGER, PARAMETER :: real8 = selected_real_kind(15)
    REAL(real4), PARAMETER :: eps4 = 1E-7
    REAL(real8), PARAMETER :: eps8 = 1D-15

    INTERFACE AssertEquals
        MODULE PROCEDURE AssertEqualsReal4
        MODULE PROCEDURE AssertEqualsReal8
        MODULE PROCEDURE AssertEqualsInt4
        MODULE PROCEDURE AssertEqualsInt8
        MODULE PROCEDURE AssertEqualsBool
        MODULE PROCEDURE AssertEqualsChar
    END INTERFACE AssertEquals
    
    PUBLIC :: eps4, eps8, int4,int8, real4, real8
    PUBLIC :: setUpTest, AssertEquals, FUnitSetUp, Fail, FUnitStatistics

    INTEGER :: nTests, nFail
CONTAINS

    SUBROUTINE setUpTest(msg)
        CHARACTER(len=*), intent(in)::msg
        print *, msg
    END SUBROUTINE
    
    SUBROUTINE FUnitSetup
        nTests = 0
        nFail = 0
    END SUBROUTINE FUnitSetup

    SUBROUTINE newTest
        nTests = nTests + 1
    END SUBROUTINE newTest

    SUBROUTINE failure
        nFail = nFail + 1
    END SUBROUTINE failure

    SUBROUTINE FUnitStatistics
        character :: c
        print *, "Tests   : ", nTests
        print *, "Failure : ", nFail
        print *, "%Success: ", (nTests-nFail)*100.0/nTests
        print *, "Press enter..."

        read(*,'(A)') c
    END SUBROUTINE FUnitStatistics

    SUBROUTINE AssertEqualsReal4(expectedValue, value, eps, location)
        REAL(kind = REAL4), intent(in) :: expectedValue, value, eps
        CHARACTER(len = *), intent(in), OPTIONAL :: location
        CALL newTest
        if (abs(expectedValue-value) .le. eps) return
        IF (present(location)) THEN
            print *, "FAILURE: Expected", expectedValue, " Got: ", value, " @ ", location
        ELSE
            print *, "FAILURE: Expected", expectedValue, " Got: ", value
        END IF
        call failure
    END SUBROUTINE AssertEqualsReal4

    SUBROUTINE AssertEqualsReal8(expectedValue, value, eps, location)
        REAL(kind = REAL8), intent(in) :: expectedValue, value, eps
        CHARACTER(len = *), intent(in), OPTIONAL :: location
        CALL newTest
        if (abs(expectedValue-value) .le. eps) return
        IF (present(location)) THEN
            print *, "FAILURE: Expected", expectedValue, " Got: ", value, " @ ", location
        ELSE
            print *, "FAILURE: Expected", expectedValue, " Got: ", value
        END IF
        call failure
    END SUBROUTINE AssertEqualsReal8

    SUBROUTINE AssertEqualsInt4(expectedValue, value, location)
        INTEGER(kind = int4), intent(in) :: expectedValue, value
        CHARACTER(len = *), intent(in), OPTIONAL :: location
        CALL newTest
        if (expectedValue .eq. value) return
        IF (present(location)) THEN
            print *, "FAILURE: Expected", expectedValue, " Got: ", value, " @ ", location
        ELSE
            print *, "FAILURE: Expected", expectedValue, " Got: ", value
        END IF
        call failure
    END SUBROUTINE AssertEqualsInt4

    SUBROUTINE AssertEqualsInt8(expectedValue, value, location)
        INTEGER(kind=int8), intent(in) :: expectedValue, value
        CHARACTER(len=*), intent(in), OPTIONAL :: location
        CALL newTest
        if(expectedValue .eq. value) return
        IF(present(location)) THEN
            print *, "FAILURE: Expected",expectedValue, " Got: ", value, " @ ", location
        ELSE
            print *, "FAILURE: Expected",expectedValue, " Got: ", value
        END IF
        call failure
    END SUBROUTINE AssertEqualsInt8

    SUBROUTINE AssertEqualsBool(expectedValue, value, location)
        LOGICAL, intent(in) :: expectedValue, value
        CHARACTER(len=*), intent(in), OPTIONAL :: location
        CALL newTest
        if(expectedValue .eqv. value) return
        IF(present(location)) THEN
            print *, "FAILURE: Expected",expectedValue, " Got: ", value, " @ ", location
        ELSE
            print *, "FAILURE: Expected",expectedValue, " Got: ", value
        END IF
        call failure
    END SUBROUTINE AssertEqualsBool

     SUBROUTINE AssertEqualsChar(expectedValue, value, location)
        CHARACTER(len=*), intent(in) :: expectedValue, value
        CHARACTER(len=*), intent(in), OPTIONAL :: location
        CALL newTest
        if(expectedValue .eq. value) return
        IF(present(location)) THEN
            print *, "FAILURE: Expected",expectedValue, " Got: ", value, " @ ", location
        ELSE
            print *, "FAILURE: Expected",expectedValue, " Got: ", value
        END IF
        call failure
    END SUBROUTINE AssertEqualsChar

   SUBROUTINE fail(msg)
        CHARACTER(LEN=*), intent(in) :: msg
        print *,"FAILURE: ", msg
        CALL failure
   END SUBROUTINE fail
END MODULE FUnit
