/*
 * Quad.java - Quad element
 *
 * Klaus Ramstöck, 2011-03-12 klra67@freenet.de
 *
 * Copyright (C) 2010 Klaus Ramstöck klra67@freenet.de
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 */


package Alex.Elements;

/**
 * Quad- class to represent quad elements
 * @author Klaus Ramstöck klra67@freenet.de
 */
public class Quad extends Element{

    /**
     * Construct quad by propertyid, partid and node array
     * @param myId
     * @param myPropertyId
     * @param myPartId
     * @param n[4]
     */
    public Quad(long myId, long myPropertyId, long myPartId, long n[]){
        super(myId, myPropertyId, myPartId, n);
        if(n.length != 4){
            throw new IllegalArgumentException();
        }
    }

    /**
     * compute warpage of quad as sin of angle between normals on two corners
     * @param Nodes n - Nodes container
     * @return double warp 0 for unwarped elements, >0 otherwise
     */
    double warp(Nodes n){
        Vector v1 = n.get(this.n1()).distV(n.get(this.n2()));
        Vector v2 = n.get(this.n1()).distV(n.get(this.n4()));
        Vector v3 = n.get(this.n3()).distV(n.get(this.n4()));
        Vector v4 = n.get(this.n3()).distV(n.get(this.n2()));
        v1 = v1.cross(v2); v1.normalize();
        v3 = v3.cross(v4); v3.normalize();
        return v1.cross(v3).norm();
    }
}
