/*
 * Vector.java - 3D vector library
 *
 * Klaus Ramstöck, 2011-03-12 klra67@freenet.de
 *
 * Copyright (C) 2010 Klaus Ramstöck klra67@freenet.de
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 */


package Alex.Elements;

/**
 * 3D Vector library
 * @author Klaus Ramstöck klra67@freenet.de
 */
public class Vector {

    public float x,y,z;

    /**
     * Construct null vector
     */
    public Vector(){}

    /**
     * Construct vector to x,y,z
     * @param xx
     * @param yy
     * @param zz
     */
    public Vector(float xx, float yy, float zz){
        x=xx; y=yy; z=zz;
    }

    /**
     * Construct copy of vector v
     * @param v
     */
    public Vector(Vector v){
        x=v.x; y=v.y; z=v.z;
    }

    /**
     *
     * @return String representation (x,y,z) of vector
     */
    public String toString(){
        String str = new String("(");
        str += x + "," + y + "," + z +")";
        return str;
    }

    /**
     * add vector b to this
     * @param b
     * @return this
     */
    public Vector add(Vector b){
        x += b.x; y += b.y; z+=b.z;
        return this;
    }

    /**
     * subtract vector b from this
     * @param b
     * @return this
     */
    public Vector sub(Vector b){
        x -= b.x; y -= b.y; z-=b.z;
        return this;
    }

    /**
     * compute dot product with vector b
     * @param b
     * @return
     */
    public double dot(Vector b){
        return x*b.x + y*b.y + z*b.z;
    }

    /**
     * compute cross product with vector b
     * @param b
     * @return new Vector cross
     */
    public Vector cross(Vector b){
        Vector res = new Vector();
        res.x = y*b.z-z*b.y;
        res.y = -(x*b.z - z*b.x);
        res.z = x*b.y - y*b.x;
       return res;
    }

    /**
     * compute distance to vector b
     * @param b
     * @return distance
     */
    public double dist(Vector b){
        double dx, dy, dz;
        dx = x-b.x;
        dy = y-b.y;
        dz = z-b.z;
        return Math.sqrt(dx*dx+dy*dy+dz*dz);
    }

    /**
     *
     * @return length of this
     */
    public double norm(){
        double xx,yy,zz;
        xx=x*x; yy=y*y; zz=z*z;
        return Math.sqrt(xx+yy+zz);
    }

    /**
     * Normalize this
     * @return this
     */
    public Vector normalize(){
        double len = this.norm();
        if(len>0.){
            x/=len; y/=len; z/=len;
        }
        return this;
    }

    /**
     * Scale this
     * @param float s
     * @return this
     */
    public Vector scale(float s){
        x*=s; y*=s; z*=s;
        return this;
    }
}
